%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Script to support Q/A of hippocampal subfields
% visualization and QC, 
% Initial version by Neda Jahanshad
% Modifications 12/16 - Philipp Sämann 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
warning off all
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ADAPT AS REQUIRED
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
subject_folder      = '/data2'; % /enigma-subfields/Whalen_paper_after_flipcorrection';
dirOUT_general      = '/data2/QC'; % /enigma-subfields/Whalen_paper_after_flipcorrection/QC_subfields';
HIPPOmap            = load('/home/philipp/SUBFIELD_scripts_part_II/HippoColors_v6.0.txt');
fid                 = fopen('/data2/enigma-subfields/Whalen_paper_after_flipcorrection/list.txt','r');
columnwidth         = 8; % maximum length of subject name
transparency        = 0.80; % transparency of overlay of regions or fissure [max 1=opaque]
transparency_aseg   = 0.15; % transparency of aseg based hippocampus; if negative: NO aseg overlay
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for fissureonly = 0:1
    
    % Get copy of colormap
    if fissureonly == 1
        HIPPOmap(11,:) = [255,255,0];
        HIPPOmap(13,:) = [0,0,255]; % add a color for the aseg hipp
    end
    
    % Divide color map by 256
    HIPPOmap=HIPPOmap./256;
    
    % Read in the subject list & fill up blanks as required
    flag = 1;
    n    = 1;
    while flag == 1
        if feof(fid)
            flag=-1;
        else
            tmp = fgets(fid);
            while size(tmp,2) < columnwidth; tmp = [tmp,' '];end
            subjectID(n,:) = tmp; n=n+1;
        end
    end
    
    % create general output directory, ignore warnings if already existent
    mkdir(dirOUT_general);
    
    % start of loop round subjects
    for master = 1:size(subjectID,1) % reduce loop range for test purposes if desired, e. g. 1:2
        
        % Prepare paths and directories
        tmp_subjectID = deblank(subjectID(master,:));
        currentdir = [subject_folder,'/',tmp_subjectID,'/mri/assembly'];
        cd (currentdir);
        dirOS = [dirOUT_general,'/',deblank(subjectID(master,:)),'/'];
        mkdir (dirOS);
        
        % Read in 4 regions
        bg_left  = load_nii('nu_left.nii');  bg_left_image  = bg_left.img;
        bg_right = load_nii('nu_right.nii'); bg_right_image = bg_right.img;
        subregions_left  = load_nii('left_discreteLabels.nii');  subregions_left_image  = subregions_left.img;
        subregions_right = load_nii('right_discreteLabels.nii'); subregions_right_image = subregions_right.img;
        
        if fissureonly
            tmp = load_nii('aparc+aseg_highres_L.nii'); aseg_image_L = tmp.img;
            tmp = load_nii('aparc+aseg_highres_R.nii'); aseg_image_R = tmp.img;
        end
        
        message = ['Loaded 6 volumes of ', currentdir]; disp(message);
        
        % Start hemisphere loop
        sides = {'left'; 'right'};
        sideshorts = {'.L'; '.R'};
        
        for si = 1:2
            side = sides{si}; sideshort = sideshorts{si};
            clear T1 all_over
            if si==1; T1=double(bg_left_image);end
            if si==2; T1=double(bg_right_image);end
            
            % separate the different subregion codes into running 1...12
            clear codes names
            codes = [203,204,205,206,208,209,210,211,212,214,215,226];
            prefix = ''; if fissureonly; codes = [-1, -1,-1,-1,-1,-1,-1,-1,-1,-1,215,-1]; prefix = 'fis_'; end
            
            % variable names not needed yet listed here for completeness:
            names = ['parasubibulum     ';
                'presubiculum      ';
                'subiculum         ';
                'CA1               ';
                'CA3               ';
                'CA4               ';
                'GC_ML_DG          ';
                'HATA              ';
                'fimbria           ';
                'molecular_layer_HP';
                'fissure           ';
                'Hippocampal_tail  ';];
            
            all_over = T1.*0; % create empty matrix of same dimension as T1
            
            % if fissure is switched on: include hippocampal shape from aseg
            if fissureonly && transparency_aseg > 0
                if si==1;all_over(find(aseg_image_L==17))=13;end
                if si==2;all_over(find(aseg_image_R==53))=13;end
            end
            
            % translate region codes into color codes running from 1 to 12
            for j=1:12
                if si==1;all_over(find(subregions_left_image==codes(j)))=j;end
                if si==2;all_over(find(subregions_right_image==codes(j)))=j;end
            end
            
            overlay = double(all_over);
            mask = overlay.*0; % empty starting matrix
            indx = find(all_over~=0);
            mask(indx) = transparency; % voxels ovewritten by transparency factor
            
            % if fissure make aseg-based overlay more transparent
            if fissureonly && transparency_aseg > 0
                indx2 = find(all_over==13);
                mask(indx2) = transparency_aseg;
            end
            
            if si==1
                Nx = bg_left.hdr.dime.dim(2);
                Ny = bg_left.hdr.dime.dim(3);
                Nz = bg_left.hdr.dime.dim(4);
                dx = bg_left.hdr.dime.pixdim(2);
                dy = bg_left.hdr.dime.pixdim(3);
                dz = bg_left.hdr.dime.pixdim(4);
            else
                Nx = bg_right.hdr.dime.dim(2);
                Ny = bg_right.hdr.dime.dim(3);
                Nz = bg_right.hdr.dime.dim(4);
                dx = bg_right.hdr.dime.pixdim(2);
                dy = bg_right.hdr.dime.pixdim(3);
                dz = bg_right.hdr.dime.pixdim(4);
            end
            
            [x y z] = ind2sub([Nx Ny Nz],indx);
            
            XminXmax = quantile(x,[0 1]) ;
            YminYmax = quantile(y,[0 1]) ;
            ZminZmax = quantile(z,[0 1]) ;
            Xmin = max(XminXmax(1),3);Xmax = min(XminXmax(2),Nx-3);
            Ymin = max(YminYmax(1),3);Ymax = min(YminYmax(2),Ny-3);
            Zmin = max(ZminZmax(1),3);Zmax = min(ZminZmax(2),Nz-3);
            sliceSs = round(quantile(x,[.2 .4 .6 .8]));
            sliceCs = round(quantile(y,[.2 .4 .6 .8]));
            sliceAs = round(quantile(z,[.2 .4 .6 .8]));
            txts = {'20';'40';'60';'80'};
            
            cmin = round(quantile(T1(T1(:)>0),[0.010]));
            cmax = round(quantile(T1(T1(:)>0),[0.999]));
            
            m1 = 128; % number of color steps for T1 image
            map1 = colormap([Gray(m1); HIPPOmap]);
            
            % start to create images
            % ======================
            
            for slice = 1:4
                sliceA = sliceAs(slice);
                sliceC = sliceCs(slice);
                sliceS = sliceSs(slice);
                txt = char(txts(slice));
                
                % =====================
                %  Create axial images
                % =====================
                h = figure('Visible','on') ;
                C1 = min(m1,round((m1-1)*(T1(:,:,sliceA)-cmin)/(cmax-cmin))+1);
                C2 = overlay(:,:,sliceA) + m1;
                h(1) = image(Xmax+2:-1:Xmin-2,Ymax+2:-1:Ymin-2,transpose(T1(:,:,sliceA))); hold on;
                h(2) = image(Xmax+2:-1:Xmin-2,Ymax+2:-1:Ymin-2,transpose(overlay(:,:,sliceA)));
                set(h(2),'AlphaData',transpose(mask(:,:,sliceA)),'AlphaDataMapping','none');
                set(h(1),'CData',C1');
                set(h(2),'CData',C2');
                text('units','pixels','position',[4 9],'BackgroundColor','w','fontsize',12,'string',[tmp_subjectID,sideshort],'Interpreter','none');
                colormap(map1); axis off; axis square; hold off;
                saveas(gca,char(strcat(dirOS,'HippoSubField_set',side,'_Axial_',prefix,txt,'.png')),'png')
                close all;
                
                % =======================
                %  Create coronal images
                % =======================
                h = figure('Visible','on') ;
                C1 = min(m1,round((m1-1)*(squeeze(T1(:,sliceC,:))-cmin)/(cmax-cmin))+1);
                C2 = squeeze(overlay(:,sliceC,:)) +m1;
                h(1) = image(Xmin-2:Xmax+2,Zmax+2:-1:Zmin-2,transpose(squeeze(T1(:,sliceC,:)))); hold on;
                h(2) = image(Xmin-2:Xmax+2,Zmax+2:-1:Zmin-2,transpose(squeeze(overlay(:,sliceC,:))));
                set(h(2),'AlphaData',transpose(squeeze(mask(:,sliceC,:))),'AlphaDataMapping','none');
                set(h(1),'CData',C1');
                set(h(2),'CData',C2');
                text('units','pixels','position',[4 9],'BackgroundColor','w','fontsize',12,'string',[tmp_subjectID,sideshort],'Interpreter','none');
                colormap(map1); axis off; axis square; hold off;
                saveas(gca,char(strcat(dirOS,'HippoSubField_set',side,'_Coronal_',prefix,txt,'.png')),'png')
                close all;
                
                % ========================
                %  Create sagittal images
                % ========================
                
                h = figure('Visible','on');
                C1 = min(m1,round((m1-1)*(squeeze(T1(sliceS,:,:))-cmin)/(cmax-cmin))+1);
                C2 = squeeze(overlay(sliceS,:,:)) +m1;
                h(1) = image(Ymin-2:Ymax+2,Zmax+2:-1:Zmin-2,transpose(squeeze(T1(sliceS,:,:)))); hold on; % X >> Y
                h(2) = image(Ymin-2:Ymax+2,Zmax+2:-1:Zmin-2,transpose(squeeze(overlay(sliceS,:,:))));
                set(h(2),'AlphaData',transpose(squeeze(mask(sliceS,:,:))),'AlphaDataMapping','none');
                set(h(1),'CData',C1');
                set(h(2),'CData',C2');
                text('units','pixels','position',[4 9],'BackgroundColor','w','fontsize',12,'string',[tmp_subjectID,sideshort],'Interpreter','none');
                colormap(map1); axis off; axis square; hold off;
                saveas(gca,char(strcat(dirOS,'HippoSubField_set',side,'_Sagittal_',prefix,txt,'.png')),'png')
                close all;
                
            end
            message = ['Written out hippocampal subregional QC images for ',side,' side.']; disp(message);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % produce only T1 based images
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            if fissureonly
                
                mask(indx) = 0; % make invisible
                
                if si==1
                    Nx = bg_left.hdr.dime.dim(2);
                    Ny = bg_left.hdr.dime.dim(3);
                    Nz = bg_left.hdr.dime.dim(4);
                    dx = bg_left.hdr.dime.pixdim(2);
                    dy = bg_left.hdr.dime.pixdim(3);
                    dz = bg_left.hdr.dime.pixdim(4);
                else
                    Nx = bg_right.hdr.dime.dim(2);
                    Ny = bg_right.hdr.dime.dim(3);
                    Nz = bg_right.hdr.dime.dim(4);
                    dx = bg_right.hdr.dime.pixdim(2);
                    dy = bg_right.hdr.dime.pixdim(3);
                    dz = bg_right.hdr.dime.pixdim(4);
                end
                
                [x y z] = ind2sub([Nx Ny Nz],indx);
                
                XminXmax = quantile(x,[0 1]) ;
                YminYmax = quantile(y,[0 1]) ;
                ZminZmax = quantile(z,[0 1]) ;
                Xmin = max(XminXmax(1),3);Xmax = min(XminXmax(2),Nx-3);
                Ymin = max(YminYmax(1),3);Ymax = min(YminYmax(2),Ny-3);
                Zmin = max(ZminZmax(1),3);Zmax = min(ZminZmax(2),Nz-3);
                sliceSs = round(quantile(x,[.2 .4 .6 .8]));
                sliceCs = round(quantile(y,[.2 .4 .6 .8]));
                sliceAs = round(quantile(z,[.2 .4 .6 .8]));
                txts = {'20';'40';'60';'80'};
                
                cmin = round(quantile(T1(T1(:)>0),[.010]));
                cmax = round(quantile(T1(T1(:)>0),[.999]));
                
                m1 = 128; % number of color steps for T1 image
                map1 = colormap([Gray(m1); HIPPOmap]);
                
                % start to create images
                % ======================
                
                for slice = 1:4
                    sliceA = sliceAs(slice);
                    sliceC = sliceCs(slice);
                    sliceS = sliceSs(slice);
                    txt = char(txts(slice));
                    
                    % =====================
                    %  Create axial images
                    % =====================
                    h = figure('Visible','on') ;
                    C1 = min(m1,round((m1-1)*(T1(:,:,sliceA)-cmin)/(cmax-cmin))+1);
                    C2 = overlay(:,:,sliceA) + m1;
                    h(1) = image(Xmax+2:-1:Xmin-2,Ymax+2:-1:Ymin-2,transpose(T1(:,:,sliceA))); hold on;
                    h(2) = image(Xmax+2:-1:Xmin-2,Ymax+2:-1:Ymin-2,transpose(overlay(:,:,sliceA)));
                    set(h(2),'AlphaData',transpose(mask(:,:,sliceA)),'AlphaDataMapping','none');
                    set(h(1),'CData',C1');
                    set(h(2),'CData',C2');
                    text('units','pixels','position',[4 9],'BackgroundColor','w','fontsize',12,'string',[tmp_subjectID,sideshort],'Interpreter','none');
                    colormap(map1); axis off; axis square; hold off;
                    saveas(gca,char(strcat(dirOS,'HippoSubField_T1_set',side,'_Axial_',txt,'.png')),'png')
                    close all;
                    
                    % =======================
                    %  Create coronal images
                    % =======================
                    h = figure('Visible','on') ;
                    C1 = min(m1,round((m1-1)*(squeeze(T1(:,sliceC,:))-cmin)/(cmax-cmin))+1);
                    C2 = squeeze(overlay(:,sliceC,:)) +m1;
                    h(1) = image(Xmin-2:Xmax+2,Zmax+2:-1:Zmin-2,transpose(squeeze(T1(:,sliceC,:)))); hold on;
                    h(2) = image(Xmin-2:Xmax+2,Zmax+2:-1:Zmin-2,transpose(squeeze(overlay(:,sliceC,:))));
                    set(h(2),'AlphaData',transpose(squeeze(mask(:,sliceC,:))),'AlphaDataMapping','none');
                    set(h(1),'CData',C1');
                    set(h(2),'CData',C2');
                    text('units','pixels','position',[4 9],'BackgroundColor','w','fontsize',12,'string',[tmp_subjectID,sideshort],'Interpreter','none');
                    colormap(map1); axis off; axis square; hold off;
                    saveas(gca,char(strcat(dirOS,'HippoSubField_T1_set',side,'_Coronal_',txt,'.png')),'png')
                    close all;
                    
                    % ========================
                    %  Create sagittal images
                    % ========================
                    
                    h = figure('Visible','on');
                    C1 = min(m1,round((m1-1)*(squeeze(T1(sliceS,:,:))-cmin)/(cmax-cmin))+1);
                    C2 = squeeze(overlay(sliceS,:,:)) +m1;
                    h(1) = image(Ymin-2:Ymax+2,Zmax+2:-1:Zmin-2,transpose(squeeze(T1(sliceS,:,:)))); hold on; % X >> Y
                    h(2) = image(Ymin-2:Ymax+2,Zmax+2:-1:Zmin-2,transpose(squeeze(overlay(sliceS,:,:))));
                    set(h(2),'AlphaData',transpose(squeeze(mask(sliceS,:,:))),'AlphaDataMapping','none');
                    set(h(1),'CData',C1');
                    set(h(2),'CData',C2');
                    text('units','pixels','position',[4 9],'BackgroundColor','w','fontsize',12,'string',[tmp_subjectID,sideshort],'Interpreter','none');
                    colormap(map1); axis off; axis square; hold off;
                    saveas(gca,char(strcat(dirOS,'HippoSubField_T1_set',side,'_Sagittal_',txt,'.png')),'png')
                    close all;
                end
                message = ['Written out pure T1 images for ',side,' side.']; disp(message);
            end
        end
    end
end
